{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": 2,
   "metadata": {},
   "outputs": [
    {
     "name": "stderr",
     "output_type": "stream",
     "text": [
      "Using TensorFlow backend.\n",
      "/home/lmu/.anyenv/envs/pyenv/versions/3.7.4/envs/songpyeon/lib/python3.7/site-packages/pandas/compat/__init__.py:84: UserWarning: Could not import the lzma module. Your installed Python is incomplete. Attempting to use lzma compression will result in a RuntimeError.\n",
      "  warnings.warn(msg)\n",
      "/home/lmu/.anyenv/envs/pyenv/versions/3.7.4/envs/songpyeon/lib/python3.7/site-packages/pandas/compat/__init__.py:84: UserWarning: Could not import the lzma module. Your installed Python is incomplete. Attempting to use lzma compression will result in a RuntimeError.\n",
      "  warnings.warn(msg)\n"
     ]
    }
   ],
   "source": [
    "import warnings\n",
    "warnings.simplefilter(action='ignore', category=FutureWarning)\n",
    "from keras.layers import Input, Reshape, Dropout, Dense, Flatten\n",
    "from keras.layers import BatchNormalization, Activation, ZeroPadding2D\n",
    "from keras.layers.advanced_activations import LeakyReLU\n",
    "from keras.layers.convolutional import UpSampling2D, Conv2D\n",
    "from keras.models import Sequential, Model, load_model\n",
    "from keras.optimizers import Adam\n",
    "from keras.callbacks import ModelCheckpoint"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "metadata": {},
   "outputs": [],
   "source": [
    "import matplotlib.pyplot as plt\n",
    "import numpy as np\n",
    "\n",
    "from PIL import Image\n",
    "from tqdm import tqdm\n",
    "\n",
    "import os"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 7,
   "metadata": {},
   "outputs": [],
   "source": [
    "class SongpyeonGenerator:\n",
    "    def __init__(self,image_width,image_height,channels):\n",
    "        self.image_width = image_width\n",
    "        self.image_height = image_height\n",
    "\n",
    "        self.channels = channels\n",
    "\n",
    "        self.image_shape = (self.image_width,self.image_height,self.channels)\n",
    "\n",
    "        #Amount of randomly generated numbers for the first layer of the generator.\n",
    "        self.random_noise_dimension = 100\n",
    "\n",
    "        #Just 10 times higher learning rate would result in generator loss being stuck at 0.\n",
    "        optimizer = Adam(0.0002,0.5)\n",
    "\n",
    "        self.discriminator = self.build_discriminator()\n",
    "        self.discriminator.compile(loss=\"binary_crossentropy\",optimizer=optimizer,metrics=[\"accuracy\"])\n",
    "        self.generator = self.build_generator()\n",
    "\n",
    "        #A placeholder for the generator input.\n",
    "        random_input = Input(shape=(self.random_noise_dimension,))\n",
    "\n",
    "        #Generator generates images from random noise.\n",
    "        generated_image = self.generator(random_input)\n",
    "\n",
    "        # For the combined model we will only train the generator\n",
    "        self.discriminator.trainable = False\n",
    "\n",
    "        #Discriminator attempts to determine if image is real or generated\n",
    "        validity = self.discriminator(generated_image)\n",
    "\n",
    "        #Combined model = generator and discriminator combined.\n",
    "        #1. Takes random noise as an input.\n",
    "        #2. Generates an image.\n",
    "        #3. Attempts to determine if image is real or generated.\n",
    "        self.combined = Model(random_input,validity)\n",
    "        self.combined.compile(loss=\"binary_crossentropy\",optimizer=optimizer)\n",
    " \n",
    "    def get_training_data(self,datafolder):\n",
    "        print(\"Loading training data...\")\n",
    "\n",
    "        training_data = []\n",
    "        #Finds all files in datafolder\n",
    "        filenames = os.listdir(datafolder)\n",
    "        for filename in tqdm(filenames):\n",
    "            #Combines folder name and file name.\n",
    "            path = os.path.join(datafolder,filename)\n",
    "            #Opens an image as an Image object.\n",
    "            image = Image.open(path).convert('RGB')\n",
    "            #Resizes to a desired size.\n",
    "            image = image.resize((self.image_width,self.image_height),Image.ANTIALIAS)\n",
    "            #Creates an array of pixel values from the image.\n",
    "            pixel_array = np.asarray(image)\n",
    "\n",
    "            training_data.append(pixel_array)\n",
    "\n",
    "        #training_data is converted to a numpy array\n",
    "        training_data = np.reshape(training_data,(-1,self.image_width,self.image_height,self.channels))\n",
    "        return training_data\n",
    "\n",
    "\n",
    "    \n",
    "    def build_generator(self):\n",
    "        #Generator attempts to fool discriminator by generating new images.\n",
    "        model = Sequential()\n",
    "\n",
    "        gf = 64\n",
    "        model.add(Dense(16 * gf * self.image_width // 16 * self.image_height // 16,activation=\"relu\",input_dim=self.random_noise_dimension))\n",
    "        model.add(Reshape((self.image_width // 16, self.image_height // 16, gf * 16)))\n",
    "\n",
    "        #Four layers of upsampling, convolution, batch normalization and activation.\n",
    "        # 1. Upsampling: Input data is repeated. Default is (2,2). In that case a 4x4x256 array becomes an 8x8x256 array.\n",
    "        # 2. Convolution: If you are not familiar, you should watch this video: https://www.youtube.com/watch?v=FTr3n7uBIuE\n",
    "        # 3. Normalization normalizes outputs from convolution.\n",
    "        # 4. Relu activation:  f(x) = max(0,x). If x < 0, then f(x) = 0.\n",
    "\n",
    "\n",
    "        model.add(UpSampling2D())\n",
    "        model.add(Conv2D(gf * 8,kernel_size=3,padding=\"same\"))\n",
    "        model.add(BatchNormalization(momentum=0.8))\n",
    "        model.add(Activation(\"relu\"))\n",
    "\n",
    "        model.add(UpSampling2D())\n",
    "        model.add(Conv2D(gf * 4,kernel_size=3,padding=\"same\"))\n",
    "        model.add(BatchNormalization(momentum=0.8))\n",
    "        model.add(Activation(\"relu\"))\n",
    "\n",
    "        model.add(UpSampling2D())\n",
    "        model.add(Conv2D(gf,kernel_size=3,padding=\"same\"))\n",
    "        model.add(BatchNormalization(momentum=0.8))\n",
    "        model.add(Activation(\"relu\"))\n",
    "\n",
    "        model.add(UpSampling2D())\n",
    "        model.add(Conv2D(gf,kernel_size=3,padding=\"same\"))\n",
    "        model.add(BatchNormalization(momentum=0.8))\n",
    "        model.add(Activation(\"relu\"))\n",
    "\n",
    "\n",
    "        # Last convolutional layer outputs as many featuremaps as channels in the final image.\n",
    "        model.add(Conv2D(self.channels,kernel_size=3,padding=\"same\"))\n",
    "        # tanh maps everything to a range between -1 and 1.\n",
    "        model.add(Activation(\"tanh\"))\n",
    "\n",
    "        # show the summary of the model architecture\n",
    "        model.summary()\n",
    "\n",
    "        # Placeholder for the random noise input\n",
    "        input = Input(shape=(self.random_noise_dimension,))\n",
    "        #Model output\n",
    "        generated_image = model(input)\n",
    "\n",
    "        #Change the model type from Sequential to Model (functional API) More at: https://keras.io/models/model/.\n",
    "        return Model(input,generated_image)\n",
    "\n",
    "\n",
    "    def build_discriminator(self):\n",
    "        #Discriminator attempts to classify real and generated images\n",
    "        model = Sequential()\n",
    "\n",
    "        model.add(Conv2D(32, kernel_size=3, strides=2, input_shape=self.image_shape, padding=\"same\"))\n",
    "        #Leaky relu is similar to usual relu. If x < 0 then f(x) = x * alpha, otherwise f(x) = x.\n",
    "        model.add(LeakyReLU(alpha=0.2))\n",
    "\n",
    "        #Dropout blocks some connections randomly. This help the model to generalize better.\n",
    "        #0.25 means that every connection has a 25% chance of being blocked.\n",
    "        model.add(Dropout(0.25))\n",
    "        model.add(Conv2D(64, kernel_size=3, strides=2, padding=\"same\"))\n",
    "        #Zero padding adds additional rows and columns to the image. Those rows and columns are made of zeros.\n",
    "        model.add(ZeroPadding2D(padding=((0,1),(0,1))))\n",
    "        model.add(BatchNormalization(momentum=0.8))\n",
    "        model.add(LeakyReLU(alpha=0.2))\n",
    "\n",
    "        model.add(Dropout(0.25))\n",
    "        model.add(Conv2D(128, kernel_size=3, strides=2, padding=\"same\"))\n",
    "        model.add(BatchNormalization(momentum=0.8))\n",
    "        model.add(LeakyReLU(alpha=0.2))\n",
    "\n",
    "        model.add(Dropout(0.25))\n",
    "        model.add(Conv2D(256, kernel_size=3, strides=1, padding=\"same\"))\n",
    "        model.add(BatchNormalization(momentum=0.8))\n",
    "        model.add(LeakyReLU(alpha=0.2))\n",
    "\n",
    "        model.add(Dropout(0.25))\n",
    "        model.add(Conv2D(512, kernel_size=3, strides=1, padding=\"same\"))\n",
    "        model.add(BatchNormalization(momentum=0.8))\n",
    "        model.add(LeakyReLU(alpha=0.2))\n",
    "\n",
    "        model.add(Dropout(0.25))\n",
    "        #Flatten layer flattens the output of the previous layer to a single dimension.\n",
    "        model.add(Flatten())\n",
    "        #Outputs a value between 0 and 1 that predicts whether image is real or generated. 0 = generated, 1 = real.\n",
    "        model.add(Dense(1, activation='sigmoid'))\n",
    "\n",
    "        model.summary()\n",
    "\n",
    "        input_image = Input(shape=self.image_shape)\n",
    "\n",
    "        #Model output given an image.\n",
    "        validity = model(input_image)\n",
    "\n",
    "        return Model(input_image, validity)\n",
    "\n",
    "    def train(self, datafolder ,epochs,batch_size,save_images_interval, checkpoint_path, isLoad = False):\n",
    "        checkpoint_dir = os.path.dirname(checkpoint_path)\n",
    "        generator_checkpoint_path = checkpoint_path+'/generator.ckpt'\n",
    "        discriminator_checkpoint_path = checkpoint_path+'/discriminator.ckpt'\n",
    "        #Get the real images\n",
    "        training_data = self.get_training_data(datafolder)\n",
    "\n",
    "        #Map all values to a range between -1 and 1.\n",
    "        training_data = training_data / 127.5 - 1.\n",
    "\n",
    "        #Two arrays of labels. Labels for real images: [1,1,1 ... 1,1,1], labels for generated images: [0,0,0 ... 0,0,0]\n",
    "        labels_for_real_images = np.ones((batch_size,1))\n",
    "        labels_for_generated_images = np.zeros((batch_size,1))\n",
    "\n",
    "        if isLoad:\n",
    "            self.generator.load_weights(generator_checkpoint_path)\n",
    "            self.discriminator.load_weights(discriminator_checkpoint_path)\n",
    "        for epoch in range(epochs):\n",
    "            # Select a random half of images\n",
    "            indices = np.random.randint(0,training_data.shape[0],batch_size)\n",
    "            real_images = training_data[indices]\n",
    "\n",
    "            #Generate random noise for a whole batch.\n",
    "            random_noise = np.random.normal(0,1,(batch_size,self.random_noise_dimension))\n",
    "            #Generate a batch of new images.\n",
    "            generated_images = self.generator.predict(random_noise)\n",
    "\n",
    "            #Train the discriminator on real images.\n",
    "            discriminator_loss_real = self.discriminator.train_on_batch(real_images,labels_for_real_images)\n",
    "            #Train the discriminator on generated images.\n",
    "            discriminator_loss_generated = self.discriminator.train_on_batch(generated_images,labels_for_generated_images)\n",
    "            #Calculate the average discriminator loss.\n",
    "            discriminator_loss = 0.5 * np.add(discriminator_loss_real,discriminator_loss_generated)\n",
    "\n",
    "            #Train the generator using the combined model. Generator tries to trick discriminator into mistaking generated images as real.\n",
    "            generator_loss = self.combined.train_on_batch(random_noise,labels_for_real_images)\n",
    "            print (\"%d [Discriminator loss: %f, acc.: %.2f%%] [Generator loss: %f]\" % (epoch, discriminator_loss[0], 100*discriminator_loss[1], generator_loss))\n",
    "\n",
    "            if epoch % save_images_interval == 0:\n",
    "                self.save_images(epoch)\n",
    "                self.generator.save_weights(generator_checkpoint_path)\n",
    "                self.discriminator.save_weights(discriminator_checkpoint_path)\n",
    "\n",
    "        #Save the model for a later use\n",
    "        self.generator.save(\"saved_models/songpyeongenerator.h5\")\n",
    "\n",
    "\n",
    "    def save_images(self,epoch):\n",
    "        #Save 25 generated images for demonstration purposes using matplotlib.pyplot.\n",
    "        rows, columns = 5, 5\n",
    "        noise = np.random.normal(0, 1, (rows * columns, self.random_noise_dimension))\n",
    "        generated_images = self.generator.predict(noise)\n",
    "\n",
    "        generated_images = 0.5 * generated_images + 0.5\n",
    "\n",
    "        figure, axis = plt.subplots(rows, columns)\n",
    "        image_count = 0\n",
    "        for row in range(rows):\n",
    "            for column in range(columns):\n",
    "                axis[row,column].imshow(generated_images[image_count, :], cmap='spring')\n",
    "                axis[row,column].axis('off')\n",
    "                image_count += 1\n",
    "        figure.savefig(\"generated_images/generated_%d.png\" % epoch)\n",
    "        plt.close()\n",
    "\n",
    "    def generate_single_image(self,model_path,image_save_path):\n",
    "        noise = np.random.normal(0,1,(1,self.random_noise_dimension))\n",
    "        model = load_model(model_path)\n",
    "        generated_image = model.predict(noise)\n",
    "        #Normalized (-1 to 1) pixel values to the real (0 to 256) pixel values.\n",
    "        generated_image = (generated_image+1)*127.5\n",
    "        print(generated_image)\n",
    "        #Drop the batch dimension. From (1,w,h,c) to (w,h,c)\n",
    "        generated_image = np.reshape(generated_image,self.image_shape)\n",
    "\n",
    "        image = Image.fromarray(generated_image,\"RGB\")\n",
    "        image.save(image_save_path)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {
    "scrolled": true
   },
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "WARNING:tensorflow:From /home/lmu/.anyenv/envs/pyenv/versions/3.7.4/envs/songpyeon/lib/python3.7/site-packages/keras/backend/tensorflow_backend.py:66: The name tf.get_default_graph is deprecated. Please use tf.compat.v1.get_default_graph instead.\n",
      "\n",
      "WARNING:tensorflow:From /home/lmu/.anyenv/envs/pyenv/versions/3.7.4/envs/songpyeon/lib/python3.7/site-packages/keras/backend/tensorflow_backend.py:541: The name tf.placeholder is deprecated. Please use tf.compat.v1.placeholder instead.\n",
      "\n",
      "WARNING:tensorflow:From /home/lmu/.anyenv/envs/pyenv/versions/3.7.4/envs/songpyeon/lib/python3.7/site-packages/keras/backend/tensorflow_backend.py:4432: The name tf.random_uniform is deprecated. Please use tf.random.uniform instead.\n",
      "\n",
      "WARNING:tensorflow:From /home/lmu/.anyenv/envs/pyenv/versions/3.7.4/envs/songpyeon/lib/python3.7/site-packages/keras/backend/tensorflow_backend.py:148: The name tf.placeholder_with_default is deprecated. Please use tf.compat.v1.placeholder_with_default instead.\n",
      "\n",
      "WARNING:tensorflow:From /home/lmu/.anyenv/envs/pyenv/versions/3.7.4/envs/songpyeon/lib/python3.7/site-packages/keras/backend/tensorflow_backend.py:3733: calling dropout (from tensorflow.python.ops.nn_ops) with keep_prob is deprecated and will be removed in a future version.\n",
      "Instructions for updating:\n",
      "Please use `rate` instead of `keep_prob`. Rate should be set to `rate = 1 - keep_prob`.\n",
      "WARNING:tensorflow:From /home/lmu/.anyenv/envs/pyenv/versions/3.7.4/envs/songpyeon/lib/python3.7/site-packages/keras/backend/tensorflow_backend.py:190: The name tf.get_default_session is deprecated. Please use tf.compat.v1.get_default_session instead.\n",
      "\n",
      "WARNING:tensorflow:From /home/lmu/.anyenv/envs/pyenv/versions/3.7.4/envs/songpyeon/lib/python3.7/site-packages/keras/backend/tensorflow_backend.py:2041: The name tf.nn.fused_batch_norm is deprecated. Please use tf.compat.v1.nn.fused_batch_norm instead.\n",
      "\n",
      "Model: \"sequential_1\"\n",
      "_________________________________________________________________\n",
      "Layer (type)                 Output Shape              Param #   \n",
      "=================================================================\n",
      "conv2d_1 (Conv2D)            (None, 128, 128, 32)      896       \n",
      "_________________________________________________________________\n",
      "leaky_re_lu_1 (LeakyReLU)    (None, 128, 128, 32)      0         \n",
      "_________________________________________________________________\n",
      "dropout_1 (Dropout)          (None, 128, 128, 32)      0         \n",
      "_________________________________________________________________\n",
      "conv2d_2 (Conv2D)            (None, 64, 64, 64)        18496     \n",
      "_________________________________________________________________\n",
      "zero_padding2d_1 (ZeroPaddin (None, 65, 65, 64)        0         \n",
      "_________________________________________________________________\n",
      "batch_normalization_1 (Batch (None, 65, 65, 64)        256       \n",
      "_________________________________________________________________\n",
      "leaky_re_lu_2 (LeakyReLU)    (None, 65, 65, 64)        0         \n",
      "_________________________________________________________________\n",
      "dropout_2 (Dropout)          (None, 65, 65, 64)        0         \n",
      "_________________________________________________________________\n",
      "conv2d_3 (Conv2D)            (None, 33, 33, 128)       73856     \n",
      "_________________________________________________________________\n",
      "batch_normalization_2 (Batch (None, 33, 33, 128)       512       \n",
      "_________________________________________________________________\n",
      "leaky_re_lu_3 (LeakyReLU)    (None, 33, 33, 128)       0         \n",
      "_________________________________________________________________\n",
      "dropout_3 (Dropout)          (None, 33, 33, 128)       0         \n",
      "_________________________________________________________________\n",
      "conv2d_4 (Conv2D)            (None, 33, 33, 256)       295168    \n",
      "_________________________________________________________________\n",
      "batch_normalization_3 (Batch (None, 33, 33, 256)       1024      \n",
      "_________________________________________________________________\n",
      "leaky_re_lu_4 (LeakyReLU)    (None, 33, 33, 256)       0         \n",
      "_________________________________________________________________\n",
      "dropout_4 (Dropout)          (None, 33, 33, 256)       0         \n",
      "_________________________________________________________________\n",
      "conv2d_5 (Conv2D)            (None, 33, 33, 512)       1180160   \n",
      "_________________________________________________________________\n",
      "batch_normalization_4 (Batch (None, 33, 33, 512)       2048      \n",
      "_________________________________________________________________\n",
      "leaky_re_lu_5 (LeakyReLU)    (None, 33, 33, 512)       0         \n",
      "_________________________________________________________________\n",
      "dropout_5 (Dropout)          (None, 33, 33, 512)       0         \n",
      "_________________________________________________________________\n",
      "flatten_1 (Flatten)          (None, 557568)            0         \n",
      "_________________________________________________________________\n",
      "dense_1 (Dense)              (None, 1)                 557569    \n",
      "=================================================================\n",
      "Total params: 2,129,985\n",
      "Trainable params: 2,128,065\n",
      "Non-trainable params: 1,920\n",
      "_________________________________________________________________\n",
      "WARNING:tensorflow:From /home/lmu/.anyenv/envs/pyenv/versions/3.7.4/envs/songpyeon/lib/python3.7/site-packages/keras/optimizers.py:793: The name tf.train.Optimizer is deprecated. Please use tf.compat.v1.train.Optimizer instead.\n",
      "\n",
      "WARNING:tensorflow:From /home/lmu/.anyenv/envs/pyenv/versions/3.7.4/envs/songpyeon/lib/python3.7/site-packages/tensorflow/python/ops/nn_impl.py:180: add_dispatch_support.<locals>.wrapper (from tensorflow.python.ops.array_ops) is deprecated and will be removed in a future version.\n",
      "Instructions for updating:\n",
      "Use tf.where in 2.0, which has the same broadcast rule as np.where\n",
      "WARNING:tensorflow:From /home/lmu/.anyenv/envs/pyenv/versions/3.7.4/envs/songpyeon/lib/python3.7/site-packages/keras/backend/tensorflow_backend.py:2239: The name tf.image.resize_nearest_neighbor is deprecated. Please use tf.compat.v1.image.resize_nearest_neighbor instead.\n",
      "\n",
      "Model: \"sequential_2\"\n",
      "_________________________________________________________________\n",
      "Layer (type)                 Output Shape              Param #   \n",
      "=================================================================\n",
      "dense_2 (Dense)              (None, 262144)            26476544  \n",
      "_________________________________________________________________\n",
      "reshape_1 (Reshape)          (None, 16, 16, 1024)      0         \n",
      "_________________________________________________________________\n",
      "up_sampling2d_1 (UpSampling2 (None, 32, 32, 1024)      0         \n",
      "_________________________________________________________________\n",
      "conv2d_6 (Conv2D)            (None, 32, 32, 512)       4719104   \n",
      "_________________________________________________________________\n",
      "batch_normalization_5 (Batch (None, 32, 32, 512)       2048      \n",
      "_________________________________________________________________\n",
      "activation_1 (Activation)    (None, 32, 32, 512)       0         \n",
      "_________________________________________________________________\n",
      "up_sampling2d_2 (UpSampling2 (None, 64, 64, 512)       0         \n",
      "_________________________________________________________________\n",
      "conv2d_7 (Conv2D)            (None, 64, 64, 256)       1179904   \n",
      "_________________________________________________________________\n",
      "batch_normalization_6 (Batch (None, 64, 64, 256)       1024      \n",
      "_________________________________________________________________\n",
      "activation_2 (Activation)    (None, 64, 64, 256)       0         \n",
      "_________________________________________________________________\n",
      "up_sampling2d_3 (UpSampling2 (None, 128, 128, 256)     0         \n",
      "_________________________________________________________________\n",
      "conv2d_8 (Conv2D)            (None, 128, 128, 64)      147520    \n",
      "_________________________________________________________________\n",
      "batch_normalization_7 (Batch (None, 128, 128, 64)      256       \n",
      "_________________________________________________________________\n",
      "activation_3 (Activation)    (None, 128, 128, 64)      0         \n",
      "_________________________________________________________________\n",
      "up_sampling2d_4 (UpSampling2 (None, 256, 256, 64)      0         \n",
      "_________________________________________________________________\n",
      "conv2d_9 (Conv2D)            (None, 256, 256, 64)      36928     \n",
      "_________________________________________________________________\n",
      "batch_normalization_8 (Batch (None, 256, 256, 64)      256       \n",
      "_________________________________________________________________\n",
      "activation_4 (Activation)    (None, 256, 256, 64)      0         \n",
      "_________________________________________________________________\n",
      "conv2d_10 (Conv2D)           (None, 256, 256, 3)       1731      \n",
      "_________________________________________________________________\n",
      "activation_5 (Activation)    (None, 256, 256, 3)       0         \n",
      "=================================================================\n",
      "Total params: 32,565,315\n",
      "Trainable params: 32,563,523\n",
      "Non-trainable params: 1,792\n",
      "_________________________________________________________________\n"
     ]
    },
    {
     "name": "stderr",
     "output_type": "stream",
     "text": [
      "\r",
      "  0%|          | 0/364 [00:00<?, ?it/s]"
     ]
    },
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "Loading training data...\n"
     ]
    },
    {
     "name": "stderr",
     "output_type": "stream",
     "text": [
      "100%|██████████| 364/364 [00:07<00:00, 47.80it/s]\n",
      "/home/lmu/.anyenv/envs/pyenv/versions/3.7.4/envs/songpyeon/lib/python3.7/site-packages/keras/engine/training.py:493: UserWarning: Discrepancy between trainable weights and collected trainable weights, did you set `model.trainable` without calling `model.compile` after ?\n",
      "  'Discrepancy between trainable weights and collected trainable'\n",
      "/home/lmu/.anyenv/envs/pyenv/versions/3.7.4/envs/songpyeon/lib/python3.7/site-packages/keras/engine/training.py:493: UserWarning: Discrepancy between trainable weights and collected trainable weights, did you set `model.trainable` without calling `model.compile` after ?\n",
      "  'Discrepancy between trainable weights and collected trainable'\n"
     ]
    }
   ],
   "source": [
    "songpyeongenerator = SongpyeonGenerator(256, 256, 3)\n",
    "songpyeongenerator.train(datafolder=\"img\", epochs = 2000, batch_size=32, save_images_interval=100, checkpoint_path=\"checkpoint\", isLoad= False)\n",
    "songpyeongenerator.generate_single_image(\"saved_models/songpyeongenerator.h5\", \"test.png\")"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": []
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": []
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.7.4"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 2
}
